s/zz0002db92/ar5210_reset (zz0002db92)/g
s/zz0002dbb2/ar5211_reset (zz0002dbb2)/g
s/zz0002dbd2/ar5212_reset (zz0002dbd2)/g
s/zz0005b5a2/ar5210_detach (zz0005b5a2)/g
s/zz0005b5e2/ar5211_detach (zz0005b5e2)/g
s/zz000b6afe/ar5210_hasVEOL (zz000b6afe)/g
s/zz000b6b7e/ar5211_hasVEOL (zz000b6b7e)/g
s/zz000b6b90/ar5211_rxMonitor (zz000b6b90)/g
s/zz000b6be8/ar5210_getRxDP (zz000b6be8)/g
s/zz000b6bf8/ar5210_getTxDP (zz000b6bf8)/g
s/zz000b6c10/ar5212_rxMonitor (zz000b6c10)/g
s/zz000b6c1d/ar5210_disable (zz000b6c1d)/g
s/zz000b6c68/ar5211_getRxDP (zz000b6c68)/g
s/zz000b6c78/ar5211_getTxDP (zz000b6c78)/g
s/zz000b6c9d/ar5211_disable (zz000b6c9d)/g
s/zz000b6ce8/ar5212_getRxDP (zz000b6ce8)/g
s/zz000b6cf8/ar5212_getTxDP (zz000b6cf8)/g
s/zz000b6d1d/ar5212_disable (zz000b6d1d)/g
s/zz000b6d22/ar5210_gpioGet (zz000b6d22)/g
s/zz000b6da2/ar5211_gpioGet (zz000b6da2)/g
s/zz000b6e22/ar5212_gpioGet (zz000b6e22)/g
s/zz000b6e52/ar5212_gpioSet (zz000b6e52)/g
s/zz000b6ee8/ar5210_setRxDP (zz000b6ee8)/g
s/zz000b6ef8/ar5210_setTxDP (zz000b6ef8)/g
s/zz000b6f68/ar5211_setRxDP (zz000b6f68)/g
s/zz000b6f78/ar5211_setTxDP (zz000b6f78)/g
s/zz000b6fe8/ar5212_setRxDP (zz000b6fe8)/g
s/zz000b6ff8/ar5212_setTxDP (zz000b6ff8)/g
s/zz000ef309/ar5211_setMulticastFilterIndex (zz000ef309)/g
s/zz0016d848/ar5210_getTsf32 (zz0016d848)/g
s/zz0016d850/ar5210_getTsf64 (zz0016d850)/g
s/zz0016d948/ar5211_getTsf32 (zz0016d948)/g
s/zz0016d950/ar5211_getTsf64 (zz0016d950)/g
s/zz0016da48/ar5212_getTsf32 (zz0016da48)/g
s/zz0016da50/ar5212_getTsf64 (zz0016da50)/g
s/zz0016dedb/ar5210_resetDma (zz0016dedb)/g
s/zz0016df2c/ar5210_resetTsf (zz0016df2c)/g
s/zz0016dfdb/ar5211_resetDma (zz0016dfdb)/g
s/zz0016e02c/ar5211_resetTsf (zz0016e02c)/g
s/zz0016e12c/ar5212_resetTsf (zz0016e12c)/g
s/zz002db060/ar5210_getRfKill (zz002db060)/g
s/zz002db260/ar5211_getRfKill (zz002db260)/g
s/zz002db51c/ar5212_getRfGain (zz002db51c)/g
s/zz002db761/ar5210_dumpState (zz002db761)/g
s/zz002db961/ar5211_dumpState (zz002db961)/g
s/zz002dbc42/ar5212_radarWait (zz002dbc42)/g
s/zz002dc60b/ar5210_stopTxDma (zz002dc60b)/g
s/zz002dc80b/ar5211_stopTxDma (zz002dc80b)/g
s/zz002dca0b/ar5212_stopTxDma (zz002dca0b)/g
s/zz00547fed/ar5212_setMulticastFilterIndex (zz00547fed)/g
s/zz005b5446/ar5210_beaconInit (zz005b5446)/g
s/zz005b5846/ar5211_beaconInit (zz005b5846)/g
s/zz005b5c46/ar5212_beaconInit (zz005b5c46)/g
s/zz005b64fd/ar5210_fillTxDesc (zz005b64fd)/g
s/zz005b652b/ar5211_getCurRssi (zz005b652b)/g
s/zz005b68fd/ar5211_fillTxDesc (zz005b68fd)/g
s/zz005b6cfd/ar5212_fillTxDesc (zz005b6cfd)/g
s/zz005b6d03/ar5210_dumpEeprom (zz005b6d03)/g
s/zz005b6e30/ar5210_initPSPoll (zz005b6e30)/g
s/zz005b7103/ar5211_dumpEeprom (zz005b7103)/g
s/zz005b7230/ar5211_initPSPoll (zz005b7230)/g
s/zz005b779d/ar5210_phyDisable (zz005b779d)/g
s/zz005b7b9d/ar5211_phyDisable (zz005b7b9d)/g
s/zz005b7f9d/ar5212_phyDisable (zz005b7f9d)/g
s/zz005b80fd/ar5210_procRxDesc (zz005b80fd)/g
s/zz005b813d/ar5210_procTxDesc (zz005b813d)/g
s/zz005b84fd/ar5211_procRxDesc (zz005b84fd)/g
s/zz005b853d/ar5211_procTxDesc (zz005b853d)/g
s/zz005b88cb/ar5210_startTxDma (zz005b88cb)/g
s/zz005b88fd/ar5212_procRxDesc (zz005b88fd)/g
s/zz005b893d/ar5212_procTxDesc (zz005b893d)/g
s/zz005b8ccb/ar5211_startTxDma (zz005b8ccb)/g
s/zz005b90cb/ar5212_startTxDma (zz005b90cb)/g
s/zz00b6c6bc/ar5210_getRxFilter (zz00b6c6bc)/g
s/zz00b6cebc/ar5211_getRxFilter (zz00b6cebc)/g
s/zz00b6d6bc/ar5212_getRxFilter (zz00b6d6bc)/g
s/zz00b6d923/ar5212_getSlotTime (zz00b6d923)/g
s/zz00b6e27a/ar5211_gpioSetIntr (zz00b6e27a)/g
s/zz00b6ea7a/ar5212_gpioSetIntr (zz00b6ea7a)/g
s/zz00b6f161/ar5210_setLedState (zz00b6f161)/g
s/zz00b6f6bc/ar5210_setRxFilter (zz00b6f6bc)/g
s/zz00b6f961/ar5211_setLedState (zz00b6f961)/g
s/zz00b6febc/ar5211_setRxFilter (zz00b6febc)/g
s/zz00b70161/ar5212_setLedState (zz00b70161)/g
s/zz00b706bc/ar5212_setRxFilter (zz00b706bc)/g
s/zz00b7073d/ar5210_setupRxDesc (zz00b7073d)/g
s/zz00b7077d/ar5210_setupTxDesc (zz00b7077d)/g
s/zz00b70923/ar5212_setSlotTime (zz00b70923)/g
s/zz00b70f3d/ar5211_setupRxDesc (zz00b70f3d)/g
s/zz00b70f7d/ar5211_setupTxDesc (zz00b70f7d)/g
s/zz00b7173d/ar5212_setupRxDesc (zz00b7173d)/g
s/zz00b7177d/ar5212_setupTxDesc (zz00b7177d)/g
s/zz016d87f0/ar5210_enablePSPoll (zz016d87f0)/g
s/zz016d9251/ar5210_getPowerMode (zz016d9251)/g
s/zz016d97f0/ar5211_enablePSPoll (zz016d97f0)/g
s/zz016d9d41/ar5212_getDiagState (zz016d9d41)/g
s/zz016d9efa/ar5210_gpioSetIntr (zz016d9efa)/g
s/zz016da0dd/ar5212_getBssIdMask (zz016da0dd)/g
s/zz016da251/ar5211_getPowerMode (zz016da251)/g
s/zz016dab2d/ar5212_getRateTable (zz016dab2d)/g
s/zz016db251/ar5212_getPowerMode (zz016db251)/g
s/zz016dbbfe/ar5211_gpioCfgInput (zz016dbbfe)/g
s/zz016dcbfe/ar5212_gpioCfgInput (zz016dcbfe)/g
s/zz016dd051/ar5210_setPCUConfig (zz016dd051)/g
s/zz016de051/ar5211_setPCUConfig (zz016de051)/g
s/zz016df051/ar5212_setPCUConfig (zz016df051)/g
s/zz016df251/ar5210_setPowerMode (zz016df251)/g
s/zz016df89b/ar5210_resetTxQueue (zz016df89b)/g
s/zz016dfb87/ar5210_numTxPending (zz016dfb87)/g
s/zz016e00dd/ar5212_setBssIdMask (zz016e00dd)/g
s/zz016e0251/ar5211_setPowerMode (zz016e0251)/g
s/zz016e089b/ar5211_resetTxQueue (zz016e089b)/g
s/zz016e0b87/ar5211_numTxPending (zz016e0b87)/g
s/zz016e0dfd/ar5210_setupXTxDesc (zz016e0dfd)/g
s/zz016e109b/ar5210_setupTxQueue (zz016e109b)/g
s/zz016e1251/ar5212_setPowerMode (zz016e1251)/g
s/zz016e189b/ar5212_resetTxQueue (zz016e189b)/g
s/zz016e1b87/ar5212_numTxPending (zz016e1b87)/g
s/zz016e1dfd/ar5211_setupXTxDesc (zz016e1dfd)/g
s/zz016e209b/ar5211_setupTxQueue (zz016e209b)/g
s/zz016e2dfd/ar5212_setupXTxDesc (zz016e2dfd)/g
s/zz016e309b/ar5212_setupTxQueue (zz016e309b)/g
s/zz016e448a/ar5210_writeAssocid (zz016e448a)/g
s/zz016e548a/ar5211_writeAssocid (zz016e548a)/g
s/zz016e648a/ar5212_writeAssocid (zz016e648a)/g
s/zz02dafdfd/ar5210_getMacAddress (zz02dafdfd)/g
s/zz02db135d/ar5211_getDefAntenna (zz02db135d)/g
s/zz02db13ed/ar5210_enableReceive (zz02db13ed)/g
s/zz02db170a/ar5210_getRandomSeed (zz02db170a)/g
s/zz02db1763/ar5210_getInterrupts (zz02db1763)/g
s/zz02db1ab2/ar5212_getCTSTimeout (zz02db1ab2)/g
s/zz02db1bf0/ar5210_disablePSPoll (zz02db1bf0)/g
s/zz02db1dfd/ar5211_getMacAddress (zz02db1dfd)/g
s/zz02db2851/ar5212_getAckCTSRate (zz02db2851)/g
s/zz02db31b2/ar5212_getAckTimeout (zz02db31b2)/g
s/zz02db335d/ar5212_getDefAntenna (zz02db335d)/g
s/zz02db33ed/ar5211_enableReceive (zz02db33ed)/g
s/zz02db370a/ar5211_getRandomSeed (zz02db370a)/g
s/zz02db3763/ar5211_getInterrupts (zz02db3763)/g
s/zz02db3875/ar5212_getCapability (zz02db3875)/g
s/zz02db3bf0/ar5211_disablePSPoll (zz02db3bf0)/g
s/zz02db3dfd/ar5212_getMacAddress (zz02db3dfd)/g
s/zz02db53ed/ar5212_enableReceive (zz02db53ed)/g
s/zz02db5763/ar5212_getInterrupts (zz02db5763)/g
s/zz02db79ae/ar5211_gpioCfgOutput (zz02db79ae)/g
s/zz02db99ae/ar5212_gpioCfgOutput (zz02db99ae)/g
s/zz02dbd35d/ar5211_setDefAntenna (zz02dbd35d)/g
s/zz02dbd763/ar5210_setInterrupts (zz02dbd763)/g
s/zz02dbdab2/ar5212_setCTSTimeout (zz02dbdab2)/g
s/zz02dbe851/ar5212_setAckCTSRate (zz02dbe851)/g
s/zz02dbebbd/ar5212_reqTxIntrDesc (zz02dbebbd)/g
s/zz02dbf1b2/ar5212_setAckTimeout (zz02dbf1b2)/g
s/zz02dbf35d/ar5212_setDefAntenna (zz02dbf35d)/g
s/zz02dbf763/ar5211_setInterrupts (zz02dbf763)/g
s/zz02dbf875/ar5212_setCapability (zz02dbf875)/g
s/zz02dbfc1d/ar5212_setDecompMask (zz02dbfc1d)/g
s/zz02dbfdfd/ar5212_setMacAddress (zz02dbfdfd)/g
s/zz02dc1763/ar5212_setInterrupts (zz02dc1763)/g
s/zz05b6c6db/ar5212_getTxIntrQueue (zz05b6c6db)/g
s/zz05b701e0/ar5210_perCalibration (zz05b701e0)/g
s/zz05b741e0/ar5211_perCalibration (zz05b741e0)/g
s/zz05b781e0/ar5212_perCalibration (zz05b781e0)/g
s/zz05b79b1b/ar5210_releaseTxQueue (zz05b79b1b)/g
s/zz05b7db1b/ar5211_releaseTxQueue (zz05b7db1b)/g
s/zz05b81b1b/ar5212_releaseTxQueue (zz05b81b1b)/g
s/zz05b8aaed/ar5210_stopDmaReceive (zz05b8aaed)/g
s/zz05b8c2ed/ar5210_stopPcuReceive (zz05b8c2ed)/g
s/zz05b8c65a/ar5212_procMibEvent (zz05b8c65a)/g
s/zz05b8eaed/ar5211_stopDmaReceive (zz05b8eaed)/g
s/zz05b902ed/ar5211_stopPcuReceive (zz05b902ed)/g
s/zz05b92aed/ar5212_stopDmaReceive (zz05b92aed)/g
s/zz05b942ed/ar5212_stopPcuReceive (zz05b942ed)/g
s/zz067d0c47/ar5212_getChanNoise (zz067d0c47)/g
s/zz06c8dd09/ar5210_isHwCipherSupported (zz06c8dd09)/g
s/zz06d0dd09/ar5211_isHwCipherSupported (zz06d0dd09)/g
s/zz06d95eb6/ar5210_getWirelessModes (zz06d95eb6)/g
s/zz06d9dd93/ar5212_getAntennaSwitch (zz06d9dd93)/g
s/zz06da5eb6/ar5211_getWirelessModes (zz06da5eb6)/g
s/zz06de460e/ar5210_setKeyCacheEntry (zz06de460e)/g
s/zz06df460e/ar5211_setKeyCacheEntry (zz06df460e)/g
s/zz06df4dd1/ar5210_setTxPowerLimit (zz06df4dd1)/g
s/zz06dfdd93/ar5212_setAntennaSwitch (zz06dfdd93)/g
s/zz06dff44a/ar5212_setKeyCacheEntry (zz06dff44a)/g
s/zz06e00b8a/ar5212_setCoverageClass (zz06e00b8a)/g
s/zz06e04dd1/ar5211_setTxPowerLimit (zz06e04dd1)/g
s/zz06f232cc/ar5210_setKeyCacheEntryMac (zz06f232cc)/g
s/zz06f717e7/ar5210_setRegulatoryDomain (zz06f717e7)/g
s/zz06fa32cc/ar5211_setKeyCacheEntryMac (zz06fa32cc)/g
s/zz06feaf60/ar5212_setKeyCacheEntryMac (zz06feaf60)/g
s/zz06ff17e7/ar5211_setRegulatoryDomain (zz06ff17e7)/g
s/zz0703947b/ar5212_setRegulatoryDomain (zz0703947b)/g
s/zz0b68e98c/ar5210_isInterruptPending (zz0b68e98c)/g
s/zz0b6c2325/ar5210_getKeyCacheSize (zz0b6c2325)/g
s/zz0b6ca325/ar5211_getKeyCacheSize (zz0b6ca325)/g
s/zz0b6ce98c/ar5211_isInterruptPending (zz0b6ce98c)/g
s/zz0b6d2325/ar5212_getKeyCacheSize (zz0b6d2325)/g
s/zz0b6d9eff/ar5212_getTxQueueProps (zz0b6d9eff)/g
s/zz0b6f50b8/ar5212_isInterruptPending (zz0b6f50b8)/g
s/zz0b6fd363/ar5212_setBeaconTimers (zz0b6fd363)/g
s/zz0b709d02/ar5212_setTxPowerLimit (zz0b709d02)/g
s/zz0b709eff/ar5212_setTxQueueProps (zz0b709eff)/g
s/zz0b711aed/ar5210_startPcuReceive (zz0b711aed)/g
s/zz0b719aed/ar5211_startPcuReceive (zz0b719aed)/g
s/zz0b721aed/ar5212_startPcuReceive (zz0b721aed)/g
s/zz0b7b8368/ar5210_setStationBeaconTimers (zz0b7b8368)/g
s/zz0b7c7741/ar5210_setMulticastFilter (zz0b7c7741)/g
s/zz0b7d4612/ar5210_resetKeyCacheEntry (zz0b7d4612)/g
s/zz0b7f8368/ar5211_setStationBeaconTimers (zz0b7f8368)/g
s/zz0b807741/ar5211_setMulticastFilter (zz0b807741)/g
s/zz0b814612/ar5211_resetKeyCacheEntry (zz0b814612)/g
s/zz0b81ea94/ar5212_setStationBeaconTimers (zz0b81ea94)/g
s/zz0b82de6d/ar5212_setMulticastFilter (zz0b82de6d)/g
s/zz0b83ad3e/ar5212_resetKeyCacheEntry (zz0b83ad3e)/g
s/zz0b87a8f9/ar5210_queryPSPollSupport (zz0b87a8f9)/g
s/zz0b8ba8f9/ar5211_queryPSPollSupport (zz0b8ba8f9)/g
s/zz0c4ef308/ar5210_clrMulticastFilterIndex (zz0c4ef308)/g
s/zz0ccef308/ar5211_clrMulticastFilterIndex (zz0ccef308)/g
s/zz0d14d1b0/ar5212_clrMulticastFilterIndex (zz0d14d1b0)/g
s/zz0d8b8b79/ar5210_getPendingInterrupts (zz0d8b8b79)/g
s/zz0d94cd04/ar5210_isKeyCacheEntryValid (zz0d94cd04)/g
s/zz0d9b8b79/ar5211_getPendingInterrupts (zz0d9b8b79)/g
s/zz0da484a1/ar5212_getPendingInterrupts (zz0da484a1)/g
s/zz0da4cd04/ar5211_isKeyCacheEntryValid (zz0da4cd04)/g
s/zz0dadc62c/ar5212_isKeyCacheEntryValid (zz0dadc62c)/g
s/zz0db0910e/ar5210_detectCardPresent (zz0db0910e)/g
s/zz0db2910e/ar5211_detectCardPresent (zz0db2910e)/g
s/zz0db3ed86/ar5212_detectCardPresent (zz0db3ed86)/g
s/zz0dbf2c1f/ar5210_waitForBeaconDone (zz0dbf2c1f)/g
s/zz0dc12c1f/ar5211_waitForBeaconDone (zz0dc12c1f)/g
s/zz0dc3494d/ar5210_updateMibCounters (zz0dc3494d)/g
s/zz0dc38718/ar5210_updateTxTrigLevel (zz0dc38718)/g
s/zz0dc5494d/ar5211_updateMibCounters (zz0dc5494d)/g
s/zz0dc58718/ar5211_updateTxTrigLevel (zz0dc58718)/g
s/zz0dc6a5c5/ar5212_updateMibCounters (zz0dc6a5c5)/g
s/zz0dc6e390/ar5212_updateTxTrigLevel (zz0dc6e390)/g
s/zz0df78379/ar5210_resetStaBeaconTimers (zz0df78379)/g
s/zz0e078379/ar5211_resetStaBeaconTimers (zz0e078379)/g
s/zz0e107ca1/ar5212_resetStationBeaconTimers (zz0e107ca1)/g
s/zz0f8ef308/ar5210_setMulticastFilterIndex (zz0f8ef308)/g
