#define AH_SUPPORT_AR5210	1
#define AH_SUPPORT_AR5211	1
#define AH_SUPPORT_AR5212	1
#define AH_SUPPORT_AR5416	1
#define AH_SUPPORT_5111	1
#define AH_SUPPORT_5112	1
#define AH_SUPPORT_2413	1
#define AH_SUPPORT_5413	1
#define AH_SUPPORT_2133	1
#define AH_REGOPS_FUNC 1
#define AH_ENABLE_FORCEBIAS	1
